# -*- coding: utf-8 -*-
# Copyright © 2015-2016 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

# Send the tweet
'''Send the tweet'''

# 3rd party libraries imports
import sqlalchemy
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sqlalchemy.ext.declarative import declarative_base
import tweepy

# app libraries imports
from db2twitter.wasposted import WasPosted
from db2twitter.timetosend import TimeToSend

class TwReSend(object):
    '''TwReSend class'''
    def __init__(self, cfgvalues, cliargs, tweets):
        '''Constructor for the TwReSend class'''
        self.cfgvalues = cfgvalues
        self.cliargs = cliargs
        self.tweets = tweets
        # activate the twitter api
        self.auth = tweepy.OAuthHandler(self.cfgvalues['consumer_key'],
                                        self.cfgvalues['consumer_secret'])
        self.auth.secure = True
        self.auth.set_access_token(self.cfgvalues['access_token'],
                                    self.cfgvalues['access_token_secret'])
        self.api = tweepy.API(self.auth, wait_on_rate_limit=True, wait_on_rate_limit_notify=True)
        self.main()

    def main(self):
        '''main of TwSend class'''
        for tweet in self.tweets:
            if not tweet['imagepath'] or self.cfgvalues['circlenoimage']:
                if self.cliargs.dryrun:
                    print(tweet['data'])
                else:
                    self.api.update_status(status=tweet['data'])
            else:
                if self.cliargs.dryrun:
                    print('{} | image:{}'.format(tweet['data'], tweet['imagepath']))
                else:
                    self.api.update_with_media(tweet['imagepath'], status=tweet['data'])



# -*- coding: utf-8 -*-
# Copyright © 2015-2016 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

# Send the tweet
'''Send the tweet'''

# 3rd party libraries imports
import sqlalchemy
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sqlalchemy.ext.declarative import declarative_base
import tweepy

# 3rd party libraries
from db2twitter.imagesize import ImageSize
from db2twitter.timetosend import TimeToSend
from db2twitter.wasposted import WasPosted

class TwSend(object):
    '''TwSend class'''
    def __init__(self, cfgvalues, cliargs, tweets, session, dbvalues):
        '''Constructor for the TwBuild class'''
        _, self.lastinsertid, self.firstrun = dbvalues
        self.alreadysent = False
        self.cfgvalues = cfgvalues
        self.cliargs = cliargs
        self.tweets = tweets
        # activate the twitter api
        self.auth = tweepy.OAuthHandler(self.cfgvalues['consumer_key'],
                                        self.cfgvalues['consumer_secret'])
        self.auth.secure = True
        self.auth.set_access_token(self.cfgvalues['access_token'],
                                    self.cfgvalues['access_token_secret'])
        self.api = tweepy.API(self.auth, wait_on_rate_limit=True, wait_on_rate_limit_notify=True)
        # activate the sqlite db
        self.session = session
        self.main()

    def main(self):
        '''main of TwSend class'''
        for tweet in self.tweets:
            self.alreadysent = False
            # store the tweets in the sqlite3 database
            if tweet['withimage']:
                self.storetweet(tweet['data'], tweet['imagepath'])
            else:
                self.storetweet(tweet['data'], '')
            # if it is the first run, dont send all the tweets
            if not self.firstrun:
                tts = TimeToSend(self.cfgvalues)
                # are date and time ok to send the tweet?
                if tts.sendthetweet:
                    # was the tweet already sent?
                    if not self.alreadysent:
                        if tweet['withimage']:
                            # dry run mode
                            if self.cliargs.dryrun:
                                print('{} | image:{}'.format(tweet['data'], tweet['imagepath']))
                            else:
                                self.api.update_with_media(tweet['imagepath'], status=tweet['data'])
                        else:
                            # dry run mode
                            if self.cliargs.dryrun:
                                print(tweet['data'])
                            else:
                                self.api.update_status(status=tweet['data'])


    def storetweet(self, tweet, tweetimage):
        '''store a tweet in sqlite database'''
        newtweet = WasPosted(tweet=tweet, tweetimage=tweetimage, lastinsertid=self.lastinsertid)
        try:
            self.session.add(newtweet)
            self.session.commit()
        except (sqlalchemy.exc.IntegrityError) as err:
            print(err)
            print('tweet already sent')
            self.session.rollback()
            self.alreadysent = True



# -*- coding: utf-8 -*-
# Copyright © 2015-2016 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

# Is it time to send the tweet
'''Is it time to send the tweet'''

# standard libraries imports
import datetime
import sys

class TimeToSend(object):
    '''TimeToSend class'''
    def __init__(self, cfgvalues):
        '''Constructor for the TimeToSend class'''
        self.weekdays = ['mon','tue','wed','thu','fri','sat','sun']
        self.translateweekdays = {0: 'mon',
                                    1: 'tue',
                                    2: 'wed',
                                    3: 'thu',
                                    4: 'fri',
                                    5: 'sat',
                                    6: 'sun'}
        self.dayhours = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24]
        activehours = [i for i in cfgvalues['hours'].split(',') if i != '']
        activedays = [i for i in cfgvalues['days'].split(',') if i != '']
        self.sendthesedays = []
        self.sendthesehours = []
        # get the days while sending tweets is authorized
        for day in activedays:
            if '-' in day:
                dstart, dend = day.split('-')
                gendays = (i for i in self.weekdays[self.weekdays.index(dstart):self.weekdays.index(dend)+1])
                for x in gendays: self.sendthesedays.append(x)
            else:
                self.sendthesedays.append(day)
        # get the hours while sending tweets is authorized
        for hour in activehours:
            if '-' in hour:
                hstart, hend = hour.split('-')
                for k in [hstart, hend]:
                    if int(k) < 0:
                        print('db2twitter.ini config file has a wrong value : hours should be >= 0')
                        sys.exit(0)
                    if int(k) > 23:
                        print('db2twitter.ini config file has a wrong value : hours should be <= 23')
                        sys.exit(0)
                genhours = (i for i in self.dayhours[self.dayhours.index(int(hstart)):self.dayhours.index(int(hend)+1)])
                for y in genhours:
                    self.sendthesehours.append(int(y))
            else:
                if int(hour) < 0:
                    print('db2twitter.ini config file has a wrong value : hours should be >= 0')
                    sys.exit(0)
                if int(hour) > 23:
                    print('db2twitter.ini config file has a wrong value : hours should be <= 23')
                    sys.exit(0)
                self.sendthesehours.append(int(hour))

    @property
    def sendthetweet(self):
        '''main of TimeToSend class'''
        #get the current date
        currentdate =  self.getdate()
        currentweekday = self.translateweekdays[currentdate.weekday()]
        currenthour = currentdate.hour
        isgoodday = False
        isgoodhour = False
        if currentweekday in self.sendthesedays:
            isgoodday = True
        if currenthour in self.sendthesehours:
            isgoodhour = True
        if isgoodday and isgoodhour:
            return True
        else:
            return False

    def getdate(sel):
        '''get the current date and time'''
        return datetime.datetime.now()




# -*- coding: utf-8 -*-
# Copyright © 2015-2016 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

# Check the size of an image
'''Check the size of an image'''

# 3rd party libraries imports
from PIL import Image

class ImageSize(object):
    '''ImageSize class'''
    def __init__(self, imagepath):
        '''Constructor of the ImageSize class'''
        self.imagepath = imagepath
        self.iswidthok = False
        self.isheightok = False
        self.width = 0
        self.height = 0
        self.twittermin = 4
        self.twittermax = 8192
        self.imagesizeok = False
        self.main()

    def main(self):
        '''Main of ImageSize class'''
        with Image.open(self.imagepath) as im:
            self.width, self.height = im.size
        # control the width - should be >= 4 and <= 8192
        if self.width >= self.twittermin and self.width <= self.twittermax:
            self.iswidthok = True
        else:
            self.iswidthok = False
        # control the height - should be >= 4 and <= 8192
        if self.height >= self.twittermin and self.height <= self.twittermax:
            self.isheightok = True
        else:
            self.isheightok = False
        # prepare return value
        if self.iswidthok and self.isheightok:
            self.imagesizeok = True
        else:
            self.imagesizeok = False

    @property
    def sendtotwitter(self):
        '''Return boolean if image size is ok'''
        return self.imagesizeok



# -*- coding: utf-8 -*-
# Copyright © 2015-2016 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

# Got data about the already sent tweets
'''Got data about the already sent tweets'''

# 3rs party libraries imports
import sqlalchemy
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sqlalchemy.ext.declarative import declarative_base

# app libraries imports
from db2twitter.wasposted import WasPosted

class SentTweets(object):
    '''SentTweets class'''
    def __init__(self, cfgvalues):
        '''Constructor for the TwBuild class'''
        self.cfgvalues = cfgvalues
        # activate the sqlite db
        engine = create_engine('sqlite:///{}'.format(self.cfgvalues['sqlitepath']))
        tmpsession = sessionmaker(bind=engine)
        session = tmpsession()
        self.session = session
        WasPosted.metadata.create_all(engine)

    @property
    def sqlitesession(self):
        '''Get the current session to the already sent tweets sqlite3 database'''
        return self.session



# -*- coding: utf-8 -*-
# Copyright © 2015-2016 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/

# Get values of the configuration file
'''Get values of the configuration file'''

# standard libraries imports
import configparser
import os.path
import sys

class ConfParse(object):
    '''ConfParse class'''
    def __init__(self, pathtoconf):
        '''Constructor of the ConfParse class'''
        self.rows = {}
        self.ids = {}
        self.sqlfilter = {}
        self.images = {}
        self.imagepath =''
        self.imageprefix =''
        self.fallbackimageprefix =''
        self.noimagecircling = False
        self.pathtoconf = pathtoconf
        self.upperfirstchar = False
        self.main()

    def main(self):
        '''Main of the ConfParse class'''
        # read the configuration file
        config = configparser.ConfigParser()
        try:
            with open(self.pathtoconf) as conffile:
                config.read_file(conffile)
                if config.has_section('twitter'):
                    self.consumer_key = config.get('twitter', 'consumer_key')
                    self.consumer_secret = config.get('twitter', 'consumer_secret')
                    self.access_token = config.get('twitter', 'access_token')
                    self.access_token_secret = config.get('twitter', 'access_token_secret')
                    self.tweet = config.get('twitter', 'tweet')
                    self.hashtags = config.get('twitter', 'hashtags')
                    if 'upper_first_char' in config['twitter']:
                        self.upperfirstchar = config.getboolean('twitter', 'upper_first_char')
                if config.has_section('database'):
                    self.dbconnector = config.get('database', 'dbconnector')
                    self.dbhost = config.get('database', 'dbhost')
                    self.database = config.get('database', 'database')
                    self.dbuser = config.get('database', 'dbuser')
                    self.dbpass = config.get('database', 'dbpass')
                    self.dbtables = config.get('database', 'dbtables')
                if config.has_section('timer'):
                    self.days = config.get('timer', 'days')
                    self.hours = config.get('timer', 'hours')
            
                    alltables = self.dbtables.split(',')
                    alltables = (i for i in alltables if i !='')
                    for table in alltables:
                        if config.has_option('database', '{}_rows'.format(table)):
                            rows = config.get('database', '{}_rows'.format(table)).split(',')
                            rows = [i for i in rows if i != '']
                            self.rows[table] = rows
                        if config.has_option('database', '{}_id'.format(table)):
                            self.ids[table] = config.get('database', '{}_id'.format(table))
                        if config.has_option('database', '{}_sqlfilter'.format(table)):
                            self.sqlfilter[table] = config.get('database', '{}_sqlfilter'.format(table))
                        if config.has_option('database', '{}_image'.format(table)):
                            self.images[table] = config.getboolean('database', '{}_image'.format(table))
                if config.has_section('media'):
                    self.imagepath = config.get('media', 'image_path')
                    if 'image_prefix' in config['media']:
                        self.imageprefix = config.get('media', 'image_prefix')
                    if 'fallback_image_prefix' in config['media']:
                        self.fallbackimageprefix = config.get('media', 'fallback_image_prefix')
                if config.has_section('sqlite'):
                    self.sqlitepath = config.get('sqlite', 'sqlitepath')
                if config.has_section('circle'):
                    self.twlastnb = config.get('circle', 'last_tweets')
                    self.twbatchnb = config.get('circle', 'each_time')
                    if 'no_image' in config['circle']:
                        self.noimagecircling = config.getboolean('circle', 'no_image')
        except (configparser.Error, IOError, OSError) as err:
            print(err)
            sys.exit(1)

    @property
    def confvalues(self):
        '''get the values of the configuration file'''
        return {'consumer_key': self.consumer_key,
                'consumer_secret': self.consumer_secret,
                'access_token': self.access_token,
                'access_token_secret': self.access_token_secret,
                'tweet': self.tweet,
                'hashtags': self.hashtags,
                'upper_first_char': self.upperfirstchar,
                'dbconnector': self.dbconnector,
                'dbhost': self.dbhost,
                'database': self.database,
                'dbuser': self.dbuser,
                'dbpass': self.dbpass,
                'dbtables': self.dbtables,
                'rows': self.rows,
                'images': self.images,
                'imagepath': self.imagepath,
                'imageprefix': self.imageprefix,
                'fallbackimageprefix': self.fallbackimageprefix,
                'ids': self.ids,
                'sqlfilter': self.sqlfilter,
                'sqlitepath': self.sqlitepath,
                'days': self.days,
                'hours': self.hours,
                'circlelasttwnb': self.twlastnb,
                'circletwbatchnb': self.twbatchnb,
                'circlenoimage': self.noimagecircling}



'''WasPosted mapping for SQLAlchemy'''
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import Column, Integer, String, DateTime, Boolean

MYBASE = declarative_base()

class WasPosted(MYBASE):
    '''WasPosted mapping for SQLAlchemy'''
    __tablename__ = 'wasposted'
    twid = Column(Integer, primary_key=True)
    tweet = Column(String, unique=True)
    tweetimage = Column(String, unique=False)
    lastinsertid = Column(Integer)
    lastcircled = Column(Boolean, default=False)



# -*- coding: utf-8 -*-
# Copyright © 2015-2016 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/

# Main class
'''Main class'''

# standard libraries imports
import configparser
import os.path
import sys

# 3rd party libraries imports
from sqlalchemy import desc
import tweepy

# app libraries imports
from db2twitter.cliparse import CliParse
from db2twitter.confparse import ConfParse
from db2twitter.dbparse import DbParse
from db2twitter.senttweets import SentTweets
from db2twitter.twbuild import TwBuild
from db2twitter.twsend import TwSend
from db2twitter.getdblasttweets import GetDbLastTweets
from db2twitter.twresend import TwReSend

class Main(object):
    '''Main class'''
    def __init__(self):
        '''Constructor of the Main class'''
        self.main()

    def main(self):
        '''Main of the Main class'''
        # parse the command line
        cargs = CliParse()
        cliargs = cargs.args
        # read the configuration file
        cfgparse = ConfParse(cliargs.pathtoconf)
        cfgvalues = cfgparse.confvalues
        # get the connector to the database storing the already-sent tweets
        senttweets = SentTweets(cfgvalues)
        sqlitesession = senttweets.sqlitesession
        if cliargs.circle:
                dblasttweets = GetDbLastTweets(cfgvalues, sqlitesession)
                twresend = TwReSend(cfgvalues, cliargs, dblasttweets.lasttweets)
        else:
            # parse the database
            dbparse = DbParse(cfgvalues, sqlitesession)
            dbvalues = dbparse.dbvalues
            # prepare the tweet
            twbuild = TwBuild(cfgvalues, dbvalues[0])
            tweets = twbuild.readytotweet
            # send the tweet
            twsend = TwSend(cfgvalues, cliargs, tweets, sqlitesession, dbvalues)



# -*- coding: utf-8 -*-
# Copyright © 2015-2016 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

# CLI parsing
'''CLI parsing'''

# standard libraries imports
from argparse import ArgumentParser
import os.path
import sys

class CliParse(object):
    '''CliParse class'''
    def __init__(self):
        '''Constructor for the CliParse class'''
        self.epilog = 'For more information: https://db2twitter.readthedocs.io'
        self.description = 'db2twitter automatically extracts fields from your database, use them to feed a template of tweet and send the tweet'
        self.main()

    def main(self):
        '''main of CliParse class'''
        parser = ArgumentParser(prog='db2twitter',
                                description=self.description,
                                epilog=self.epilog)
        parser.add_argument('pathtoconf', metavar='FILE', type=str,
                           help='the path to the retweet configuration')
        parser.add_argument('-c', '--circle', action='store_true',
                           default=False, help='circling the last tweets')
        parser.add_argument('--dry-run', dest='dryrun', action='store_true',
                           default=False, help='simulate the execution, no tweet sent')
        self.cliargs = parser.parse_args()
        if not os.path.exists(self.cliargs.pathtoconf):
            sys.exit('the path you provided for db2twitter configuration file does not exist')
        if not os.path.isfile(self.cliargs.pathtoconf):
            sys.exit('the path you provided for db2twitter configuration is not a file')

    @property
    def args(self):
        '''return the cli arguments'''
        return self.cliargs



#!/usr/bin/env python3.4
# -*- coding: utf-8 -*-
# Copyright © 2015 Carl Chenet <chaica@backupcheckerproject.org>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Unit tests for db2twitter
'''Unit tests for db2twitter'''

import datetime
import sys
import unittest

from db2twitter.cliparse import CliParse
from db2twitter.confparse import ConfParse
from db2twitter.twbuild import TwBuild
from db2twitter.timetosend import TimeToSend

class TestDb2Twitter(unittest.TestCase):
    '''TestDb2Twitter class'''

    def test_getconfigfile(self):
        '''Test the CliParse class'''
        filepath = './tests.py'
        circling = False
        sys.argv[-1] = filepath
        clip = CliParse()
        self.assertEqual(clip.args, {'validpathtoconf': filepath, 'iscircling': circling})

    def test_getconfigvars(self):
        '''Test the ConfParse class'''
        confp = ConfParse('tests/getconfigvars/db2twitter.ini')
        self.assertEqual(confp.confvalues, {'consumer_key': 'qPsJvBZlE2yatbsLM4dQL1r3m',
                                            'consumer_secret': 'ixM1AvyzVJxjKmsQtmQzJ8pmB4vlhmYIO5v2XqEl0mqIdIwie5',
                                            'access_token': '2370158707-m8eN2YcfVcKGfWmVLERDGMd1UPKHVPzAQHoP7qN',
                                            'access_token_secret': 'tB74LfWy551sRTU49buGE6YXIukY74TmBK6JLECeOA5ti',
                                            'tweet': '{} recrute un {} https://www.linuxjobs.fr/jobs/{}',
                                            'hashtags': 'devops,linux,',
                                            'upper_first_char': True,
                                            'dbconnector': 'mysql+mysqlconnector',
                                            'dbhost': 'localhost',
                                            'database': 'linuxjobs',
                                            'dbuser': 'linuxjobs',
                                            'dbpass': 'xxxxxxxxx',
                                            'dbtables': 'jobs,',
                                            'ids': {},
                                            'rows': {'jobs': ['company_name','title','id']},
                                            'sqlitepath': '/var/lib/db2twitter/db2twitter.db',
                                            'sqlfilter': {},
                                            'days': 'mon-fri,',
                                            'hours': '0-11,14-17,',
                                            'circlelasttwnb': '3',
                                            'circletwbatchnb': '2',
                                            'sqlfilter': {},
                                            })

    def test_gettwbuild(self):
        '''Test the TwBuild class'''
        conf= { 'tweet': '{} recrute un {} https://www.linuxjobs.fr/jobs/{}',
                'hashtags': 'devops,linux,',
                'upper_first_char': True,}
        twbuildp = TwBuild(conf, [['machin', 'devops', 1]])
        self.assertEqual(twbuildp.readytotweet, ['Machin recrute un #devops https://www.linuxjobs.fr/jobs/1'])

    def test_timetosendhourstrue(self):
        '''Test the TimeToSend class - return True'''
        conf = { 'days': 'mon-sun,',
                #'hours': '0,1,2,3,4,5-7,8,9,10,11,12,13,14,15,16,17,18,19-23,',}
                'hours': '0-23,',}
        tts = TimeToSend(conf)
        self.assertTrue(tts.sendthetweet)

    def test_timetosendhourslisttrue(self):
        '''Test the TimeToSend class with a list of hours- return True'''
        conf = { 'days': 'mon-sun,',
                'hours': '0-23,',}
        tts = TimeToSend(conf)
        self.assertTrue(tts.sendthetweet)

    @unittest.skipIf(((datetime.datetime.now().weekday() == 6) and (datetime.datetime.now().hour == 0)), 'only executing if we are not sunday and not midnight')
    def test_timetosendhoursfalse(self):
        '''Test the TimeToSend class - return False'''
        conf = { 'days': 'sun,',
                'hours': '0,',}
        tts = TimeToSend(conf)
        self.assertFalse(tts.sendthetweet)

################################################################
#
# End of the unit tests
#
################################################################

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright © 2015-2016 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

# Build the tweet to send
'''Build the tweet to send'''

# standard libraries imports
import os.path

# app libraries imports
from db2twitter.imagesize import ImageSize

class TwBuild(object):
    '''TwBuild class'''
    def __init__(self, cfgvalues, dbvalues):
        '''Constructor for the TwBuild class'''
        self.cfgvalues = cfgvalues
        self.dbvalues = dbvalues
        self.tweets = []
        self.main()

    def main(self):
        '''main of TwBuild class'''
        # get hashtags
        if self.cfgvalues['hashtags'] != '':
            hashtags = self.cfgvalues['hashtags'].split(',')
            hashtags = [i for i in hashtags if i != '']
        for i in self.dbvalues:
            iswithimage = True
            if i['withimage']:
                # the tweet uses an image
                image = i['data'][-1]
                # lets apply a specific prefix to the image
                if self.cfgvalues['imageprefix']:
                    image = ''.join([self.cfgvalues['imageprefix'], image])
                # lets apply a specific path to the image
                if self.cfgvalues['imagepath']:
                    image = os.path.join(self.cfgvalues['imagepath'], image)
                # if the image path does not exist, fallback to no-image tweet
                if not image or not os.path.exists(image) or not os.path.isfile(image):
                    iswithimage = False
                else:
                    im = ImageSize(image)
                    if not im.sendtotwitter:
                        # try the fallback
                        if not self.cfgvalues['fallbackimageprefix']:
                            iswithimage = False
                        else:
                            fallbackimage = os.path.join(os.path.dirname(image), ''.join([self.cfgvalues['fallbackimageprefix'], os.path.basename(image)]))
                            # give up if the file of the fallback image does not exist
                            if not os.path.exists(fallbackimage):
                                iswithimage = False
                            else:
                                # check the size of the fallback image
                                fallback = ImageSize(fallbackimage)
                                if not fallback.sendtotwitter:
                                    iswithimage = False
                                else:
                                    image = fallbackimage
                if iswithimage:
                    j = self.cfgvalues['tweet'].format(*i['data'])
                    # identify and replace hashtags
                    j = j.lower()
                    for hashtag in hashtags:
                        pattern = ' ' + hashtag
                        if pattern in j.lower():
                            j = j.replace(pattern, ' #{}'.format(hashtag))
                    # uppercase for the first letter of the tweet
                    if self.cfgvalues['upper_first_char']:
                        j = j[0].upper() + j[1:]
                    self.tweets.append({'withimage': True, 'data': j, 'imagepath': image})
            if not i['withimage'] or not iswithimage:
                j = self.cfgvalues['tweet'].format(*i['data'])
                # identify and replace hashtags
                j = j.lower()
                for hashtag in hashtags:
                    pattern = ' ' + hashtag
                    if pattern in j.lower():
                        j = j.replace(pattern, ' #{}'.format(hashtag))
                # uppercase for the first letter of the tweet
                if self.cfgvalues['upper_first_char']:
                    j = j[0].upper() + j[1:]
                self.tweets.append({'withimage': False, 'data': j})

    @property
    def readytotweet(self):
        '''return the tweet ready to be sent'''
        return self.tweets



# -*- coding: utf-8 -*-
# Copyright © 2015 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

# Setup for db2twitter
'''Setup for db2twitter'''

# 3rd party libraries imports
from setuptools import setup

CLASSIFIERS = [
    'Intended Audience :: End Users/Desktop',
    'Environment :: Console',
    'License :: OSI Approved :: GNU General Public License (GPL)',
    'Operating System :: POSIX :: Linux',
    'Programming Language :: Python :: 3.4'
]

setup(
    name='db2twitter',
    version='0.6',
    license='GNU GPL v3',
    description='extract info from a database and send it to twitter',
    long_description='extract information from a database, write a tweet with them and send this tweet',
    classifiers=CLASSIFIERS,
    author='Carl Chenet',
    author_email='chaica@ohmytux.com',
    url='https://github.com/chaica/db2twitter',
    download_url='https://github.com/chaica/db2twitter',
    packages=['db2twitter'],
    scripts=['scripts/db2twitter'],
    install_requires=['tweepy', 'sqlalchemy','pillow'],
)



# -*- coding: utf-8 -*-
# Copyright © 2015-2016 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/

# DbParse class
'''DbParse class'''

# standard libraries imports
import sys

# 3rd party libraries imports
from sqlalchemy import *
from sqlalchemy.orm import create_session
from sqlalchemy.ext.declarative import declarative_base

# app libraries imports
from db2twitter.wasposted import WasPosted

class DbParse(object):
    '''DbParse class'''
    def __init__(self, cfgvalues, sqlitesession):
        '''Constructor of the DbParse class'''
        self.sqlitesession = sqlitesession
        self.mainid = ''
        self.firstrun = True
        # Create and engine and get the metadata
        Base = declarative_base()
        engine = create_engine('{}://{}:{}@{}/{}'.format(cfgvalues['dbconnector'],
            cfgvalues['dbuser'],
            cfgvalues['dbpass'],
            cfgvalues['dbhost'], 
            cfgvalues['database']))
        meta = MetaData()
        meta.reflect(bind=engine)
        
        tableobjects = []
        tableschemas = {}
        # load schemas of tables
        for table in cfgvalues['rows']:
            tableschemas[table] = Table(table, meta, autoload=True, autoload_with=engine)

        #Create a session to use the tables    
        session = create_session(bind=engine)

        # find how many new tweets to send
        sqliteobj = self.sqlitesession.query(WasPosted.lastinsertid).order_by(WasPosted.lastinsertid.desc()).first()
        if sqliteobj:
            sqlitemaxid = sqliteobj.lastinsertid

        dbtables = [i for i in cfgvalues['dbtables'].split(',') if i !='']

        if not cfgvalues['ids']:
            tableobj = session.query(tableschemas[dbtables[0]]).order_by(tableschemas[dbtables[0]].columns.id.desc()).first()
            self.lastinsertid = getattr(tableobj, 'id')
        else:
            tabrequest = '{}'.format(cfgvalues['ids'][dbtables[0]])
            tableobj = session.query(tableschemas[dbtables[0]]).order_by(getattr(tableschemas[dbtables[0]].columns, tabrequest).desc()).first()
            self.lastinsertid = getattr(tableobj, cfgvalues['ids'][dbtables[0]])

        self.tweets = []
        self.allfields = []

        if sqliteobj:
            self.firstrun = False
            for table in cfgvalues['rows']:
                # if the table does not have a user-specified row 'id'
                if not cfgvalues['ids']:
                    if cfgvalues['sqlfilter']:
                        filterrequest = '{}'.format(cfgvalues['sqlfilter'][table])
                        tableobj = session.query(tableschemas[table]).filter(tableschemas[table].columns.id > sqlitemaxid, text(filterrequest))
                    else:
                        tableobj = session.query(tableschemas[table]).filter(tableschemas[table].columns.id > sqlitemaxid)
                else:
                    tabrequest = '{}'.format(cfgvalues['ids'][table])
                    if cfgvalues['sqlfilter']:
                        filterrequest = '{}'.format(cfgvalues['sqlfilter'][table])
                        tableobj = session.query(tableschemas[table]).filter(getattr(tableschemas[table].columns, tabrequest) > sqlitemaxid, text(filterrequest))
                    else:
                        tableobj = session.query(tableschemas[table]).filter(getattr(tableschemas[table].columns, tabrequest) > sqlitemaxid)

                # ignore the None query result
                if tableobj:
                    for tweetindb in tableobj:
                        if table in cfgvalues['images']:
                            # split the different fields we need, last field is the image path
                            self.tweets.append({'withimage': True, 'data': [getattr(tweetindb, i) for i in cfgvalues['rows'][table]]})
                        else:
                            self.tweets.append({'withimage': False, 'data': [getattr(tweetindb, i) for i in cfgvalues['rows'][table]]})

        else:
            self.firstrun = True
            for table in cfgvalues['rows']:
                # if the table does not have a user-specified row 'id'
                if not cfgvalues['ids']:
                    if cfgvalues['sqlfilter']:
                        filterrequest = '{}'.format(cfgvalues['sqlfilter'][table])
                        tableobj = session.query(tableschemas[table]).filter(tableschemas[table].columns.id <= self.lastinsertid, text(filterrequest))
                    else:
                        tableobj = session.query(tableschemas[table]).filter(tableschemas[table].columns.id <= self.lastinsertid)
                else:
                    tabrequest = '{}'.format(cfgvalues['ids'][table])
                    if cfgvalues['sqlfilter']:
                        filterrequest = '{}'.format(cfgvalues['sqlfilter'])
                        tableobj = session.query(tableschemas[table]).filter(getattr(tableschemas[table].columns, tabrequest) <= self.lastinsertid, text(filterrequest))
                    else:
                        tableobj = session.query(tableschemas[table]).filter(getattr(tableschemas[table].columns, tabrequest) <= self.lastinsertid)

                # ignore the None query result
                if tableobj:
                    for tweetdb in tableobj:
                        if table in cfgvalues['images']:
                            # split the different fields we need, last field is the image path
                            self.tweets.append({'withimage': True, 'data': [getattr(tweetdb, i) for i in cfgvalues['rows'][table]]})
                        else:
                            # split the different fields we need
                            self.tweets.append({'withimage': False, 'data': [getattr(tweetdb, i) for i in cfgvalues['rows'][table]]})

        # lets quit now if nothing new to tweet
        if not self.tweets:
            sys.exit(0)

    @property
    def dbvalues(self):
        '''Database parsed values'''
        return (self.tweets, self.lastinsertid, self.firstrun)



# -*- coding: utf-8 -*-
# Copyright © 2015-2016 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/

# GetDbLastTweets class
'''GetDbLastTweets class'''

# standard libraries imports
import sys

# 3rd party libraries imports
from sqlalchemy import desc

# app libraries imports
from db2twitter.wasposted import WasPosted

class GetDbLastTweets(object):
    '''GetDbLastTweets class'''
    def __init__(self, cfgvalues, sqlitesession):
        '''Constructor of the GetDbLastTweets class'''
        self.sqlitesession = sqlitesession
        # the max last tweets we can resend
        self.twnb = int(cfgvalues['circlelasttwnb'])
        # the number of tweets we send this time
        self.twbatchnb = int(cfgvalues['circletwbatchnb'])
        self.storetweets = False
        self.tweetstoresend = []
        self.main()

    def main(self):
        '''Main of GetDbLastTweets class'''
        dbresults = self.sqlitesession.query(WasPosted).order_by(desc(WasPosted.twid)).limit(self.twnb)
        i = 0
        while self.twbatchnb != 0:
            row = dbresults[i]
            # search the last circled tweet
            if row.lastcircled:
                self.storetweets = True
                row.lastcircled = False
                self.sqlitesession.commit()
            else:
                if self.storetweets:
                    # tweets to resend
                    self.tweetstoresend.append({'data': row.tweet,'imagepath': row.tweetimage})
                    self.twbatchnb -= 1
                    if self.twbatchnb == 0:
                        row.lastcircled = True
                        self.sqlitesession.commit()
            # counter to loop the row of dbresults
            if i == (self.twnb - 1):
                # check if previous circle was found, if not begin from first element in the last tweets
                if not self.storetweets:
                    self.storetweets = True
                i = 0
            else:
                i += 1

    @property
    def lasttweets(self):
        '''Last tweets to send from the database'''
        return self.tweetstoresend
